# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.transport.base import TransportGiveUp

from driverng.protocol.acknack import ACKNACKProtocol
from driverng.utils import ETB, ETX, ACK, NACK, STX, byte2log


class Au680Protocol(ACKNACKProtocol):
    ACCEPTED_END_CHAR = [ETX, ETB]

    def __init__(self, **kwargs):
        kwargs.setdefault("checksum_type", self.CHECKSUM_BCC)
        kwargs.setdefault("max_tries", 2)
        kwargs.setdefault("timeout", 3)
        super(Au680Protocol, self).__init__(**kwargs)

    def send_trames(self, trame):
        sent = super(Au680Protocol, self).send_trames(trame)
        if not sent:
            self.logger.warning("envoi échoué , abandon")
            raise TransportGiveUp

        return sent

    def recv_trames(self):
        # On lit 1 car
        data = self.read()
        if not data:
            self.logger.warning("<stx> expected, nothing received")
            return False

        # 3 NACK possible avant que le au680 n'accepte plus notre réponse(au prochain STX)
        if data[0] == NACK:
            self.logger.warning(
                "<stx> expected, ({}) <nack> received".format(self.count_nack_receive)
            )
            return False

        # On ne se comprend pas, pas la peine d'envoyer une réponse (si on a mal comprit la demande)
        if data[0] != STX:
            self.logger.warning(
                "<stx> expected but <{}> received".format(byte2log(data))
            )
            return False

        # On lit la suite jusque END_CHAR
        if self.checksum_type == self.CHECKSUM_BCC:
            end_pos = -2
        else:
            end_pos = -1
        data += self.readline(
            break_func=lambda s: s and s[end_pos] in self.ACCEPTED_END_CHAR
        )

        # Mauvais checksum, on ne se comprend toujours pas
        if not self.check_trame(data):
            self.write(NACK)
            return False

        # on doit envoyer ack
        if self.ack:
            self.write(ACK)

        # on ignore ces trames
        payload = self.get_payload(data)
        if bytearray(payload[:2]) in (b"DE", b"DB", b"SE", b"RB", b"RE"):
            self.logger.info("data ignored {}", byte2log(payload[:2]))
            return []

        return payload
